SystemOrganization addCategory: #'Kinect-Core'!

!Point methodsFor: '*kinect-core' stamp: 'Richo 1/14/2011 13:48'!
@ aNumber
	^ Point3D x: x y: y z: aNumber! !

Point subclass: #Point3D
	instanceVariableNames: 'z'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kinect-Core'!

!Point3D class methodsFor: 'instance creation' stamp: 'Richo 1/14/2011 13:50'!
x: x y: y z: z
	^ self basicNew setX: x setY: y setZ: z! !

!Point3D methodsFor: 'comparing' stamp: 'Richo 1/14/2011 14:08'!
= aPoint3D
	(aPoint3D isKindOf: self class)
		ifTrue: [^x = aPoint3D x and: [y = aPoint3D y and: [z = aPoint3D z]]]
		ifFalse: [^false]! !

!Point3D methodsFor: 'comparing' stamp: 'Richo 1/14/2011 14:09'!
hash
	"Hash is reimplemented because = is implemented."
	^ (x hash hashMultiply bitXor: y hash) hashMultiply bitXor: z hash! !

!Point3D methodsFor: 'printing' stamp: 'Richo 1/17/2011 16:51'!
printOn: aStream
super printOn: aStream.
aStream nextPut: $@.
z negative ifTrue: [
		"Avoid ambiguous @- construct"
		aStream space].
z printOn: aStream.
! !

!Point3D methodsFor: 'private' stamp: 'Richo 1/14/2011 13:50'!
setX: aNumber setY: aNumber2 setZ: aNumber3 
	x := aNumber.
	y := aNumber2.
	z := aNumber3! !

!Point3D methodsFor: 'accessing' stamp: 'Richo 1/14/2011 13:53'!
x
^x! !

!Point3D methodsFor: 'accessing' stamp: 'Richo 1/14/2011 13:53'!
y
^y! !

!Point3D methodsFor: 'accessing' stamp: 'Richo 1/14/2011 13:53'!
z
^z! !

Object subclass: #KinectServer
	instanceVariableNames: 'oscServer process skeleton times'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kinect-Core'!

!KinectServer methodsFor: 'connection' stamp: 'Richo 1/16/2011 02:15'!
connect
	self isConnected ifTrue: [^self].
	oscServer := OSCServer listenOnPort: 7110.
	self startStepping.! !

!KinectServer methodsFor: 'connection' stamp: 'Richo 1/16/2011 02:15'!
disconnect
	self isConnected ifFalse: [^self].
	self stopStepping.
	oscServer terminate.
	oscServer := nil.! !

!KinectServer methodsFor: 'connection' stamp: 'Richo 1/16/2011 02:15'!
isConnected
	^ oscServer notNil! !

!KinectServer methodsFor: 'accessing' stamp: 'Richo 1/16/2011 22:50'!
jointNames 
^ {
	'head' -> #head.
	'neck' -> #neck.
	'l_collar' -> #leftCollar.
	'l_shoulder' -> #leftShoulder.
	'l_elbow' -> #leftElbow.
	'l_wrist' -> #leftWrist.
	'l_hand' -> #leftHand.
	'l_fingertip' -> #leftFingertip.
	'r_collar' -> #rightCollar.
	'r_shoulder' -> #rightShoulder.
	'r_elbow' -> #rightElbow.
	'r_wrist' -> #rightWrist.
	'r_hand' -> #rightHand.
	'r_fingertip' -> #rightFingertip.
	'torso' -> #torso.
	'waist' -> #waist.
	'l_hip' -> #leftHip.
	'l_knee' -> #leftKnee.
	'l_ankle' -> #leftAnkle.
	'l_foot' -> #leftFoot.
	'r_hip' -> #rightHip.
	'r_knee' -> #rightKnee.
	'r_ankle' -> #rightAnkle.
	'r_foot' -> #rightFoot
} as: Dictionary ! !

!KinectServer methodsFor: 'parsing' stamp: 'Richo 1/17/2011 16:53'!
parse: msgArray 
	| header |
	header := msgArray first.
	(header = '/joint')
		ifTrue: [^self parseJoint: msgArray].
	(header = '/new_user')
		ifTrue: [^self triggerEvent: #newUser with: msgArray second].
	(header = '/lost_user')
		ifTrue: [^self triggerEvent: #lostUser with: msgArray second].
	(header = '/pose_detected')
		ifTrue: [^self triggerEvent: #poseDetected with: msgArray second].
	(header = '/calibration_start')
		ifTrue: [^self triggerEvent: #calibrationStart with: msgArray second].
	(header = '/new_skel')
		ifTrue: [^self triggerEvent: #newSkeleton with: msgArray second].
	! !

!KinectServer methodsFor: 'parsing' stamp: 'Richo 1/17/2011 12:35'!
parseJoint: msgArray 
	| position jointName userId |
	msgArray size < 6
		ifTrue: [^ self].
	jointName := self jointNames at: msgArray second.
	userId := msgArray third.
	position := msgArray fourth @ msgArray fifth @ ((msgArray sixth / 3.5) min: 1).
	self triggerEvent: #jointUpdate withArguments: {jointName. position. userId}! !

!KinectServer methodsFor: 'stepping' stamp: 'Richo 1/17/2011 17:57'!
startStepping
	process
		ifNotNil: [process resume.
			^ self].
	process := [[self step] repeat] newProcess 
						name: self class asString;
						priority: Processor highIOPriority;
						resume! !

!KinectServer methodsFor: 'stepping' stamp: 'Richo 1/16/2011 22:52'!
step
	self parse: oscServer nextMessage.! !

!KinectServer methodsFor: 'stepping' stamp: 'Richo 1/16/2011 02:15'!
stopStepping
	process
		ifNotNil: [process terminate. process := nil]! !

Morph subclass: #KinectServerMorph
	instanceVariableNames: 'skeleton kinectServer informationMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kinect-Core'!

!KinectServerMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 1/17/2011 16:50'!
descriptionForPartsBin
	"Answer a description for use in parts bins"
	^ self partName: 	'Kinect Server'
		categories:		{'Kinect'}
		documentation:	''! !

!KinectServerMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 1/17/2011 16:50'!
includeInNewMorphMenu
	^true! !

!KinectServerMorph methodsFor: 'initialize' stamp: 'Richo 1/17/2011 16:12'!
addButtons
	| start stop |
	start := (SimpleButtonMorph new label: ' Start ';
			position: self topLeft + 10;
			color: (Color green mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 cornerStyle: #rounded;
			 borderWidth: 2;
			 actionSelector: #start;
			 target: self).
	stop := (SimpleButtonMorph new label: ' Stop ';
			position: start bottomLeft + (0@10);
			color: (Color red mixed: 0.3 with: Color blue muchLighter);
			 borderColor: #raised;
			 cornerStyle: #rounded;
			 borderWidth: 2;
			 actionSelector: #stop;
			 target: self).
	self addMorph: start; addMorph: stop.
	! !

!KinectServerMorph methodsFor: 'initialize' stamp: 'Richo 1/17/2011 16:08'!
addJoints
	skeleton := Dictionary new.
	self initialJointPositions keysAndValuesDo: [:jointName :jointPos |
		self addMorph: (skeleton at: jointName put: (self newJointMorph setNameTo: jointName capitalized))].
	self positionJoints.! !

!KinectServerMorph methodsFor: 'kinect events' stamp: 'Richo 1/17/2011 14:31'!
calibrationStart: userId
	self show: 'Calibration started'! !

!KinectServerMorph methodsFor: 'halo' stamp: 'Richo 1/17/2011 16:47'!
defersHaloToInteriorMorph: aMorph
"It only defers halo to interior if the morph is one of the skeleton's joints."
^skeleton anySatisfy: [:each | each = aMorph]! !

!KinectServerMorph methodsFor: 'initialize' stamp: 'Richo 1/16/2011 02:39'!
delete
	kinectServer disconnect.
	super delete! !

!KinectServerMorph methodsFor: 'accessing' stamp: 'Richo 1/17/2011 16:49'!
extent: newExtent
	"Updates the joint positions"
	super extent: newExtent.
	skeleton ifNotNil: [self positionJoints].! !

!KinectServerMorph methodsFor: 'accessing' stamp: 'Richo 1/17/2011 13:35'!
initialJointPositions
"Roughly imitating the calibration pose"
^{
	#rightElbow -> (0.28 @ 0.32 @ 0.44).
	#leftShoulder -> (0.59 @ 0.32 @ 0.44).
	#leftElbow -> (0.72 @ 0.32 @ 0.44).
	#rightShoulder -> (0.41 @ 0.32 @ 0.44).
	#neck -> (0.50 @ 0.32 @ 0.44).
	#rightHand -> (0.28 @ 0.18 @ 0.44).
	#rightKnee -> (0.42 @ 0.74 @ 0.44).
	#leftHand -> (0.72 @ 0.18 @ 0.44).
	#leftHip -> (0.57 @ 0.59 @ 0.44).
	#leftKnee -> (0.58 @ 0.74 @ 0.44).
	#rightFoot -> (0.40 @ 0.89 @ 0.44).
	#torso -> (0.50 @ 0.53 @ 0.44).
	#head -> (0.50 @ 0.21 @ 0.44).
	#rightHip -> (0.43 @ 0.59 @ 0.44).
	#leftFoot -> (0.60 @ 0.89 @ 0.44).
} as: Dictionary! !

!KinectServerMorph methodsFor: 'initialize' stamp: 'Richo 1/17/2011 16:18'!
initialize
	super initialize.
	self color: Color white;
		 borderColor: Color red;
		 borderWidth: 2;
		 clipSubmorphs: true; 
		 extent:  480.0@360.0.
	self initializeKinectServer; 
		 addJoints;
		 addButtons.! !

!KinectServerMorph methodsFor: 'initialize' stamp: 'Richo 1/17/2011 16:48'!
initializeKinectServer
	"Creates the Kinect server and configures its events"
	kinectServer := KinectServer new.
	kinectServer when: #jointUpdate send: #updateJoint:position:user: to: self.
	kinectServer when: #newUser send: #newUser: to: self.
	kinectServer when: #lostUser send: #lostUser: to: self.
	kinectServer when: #poseDetected send: #poseDetected: to: self.
	kinectServer when: #calibrationStart send: #calibrationStart: to: self.
	kinectServer when: #newSkeleton send: #newSkeleton: to: self.! !

!KinectServerMorph methodsFor: 'kinect events' stamp: 'Richo 1/17/2011 16:48'!
lostUser: userId
	self positionJoints; 
		 show: 'Lost user'.
	"Remove the message in 2 seconds"
	World addAlarm: #show: withArguments: #('') for: self at: (Time millisecondClockValue + 2000)! !

!KinectServerMorph methodsFor: 'initialize' stamp: 'Richo 1/17/2011 11:31'!
newJointMorph
	^ EllipseMorph new color: (Color blue alpha: 0.5); borderWidth: 0; extent: 45@45! !

!KinectServerMorph methodsFor: 'kinect events' stamp: 'Richo 1/17/2011 14:31'!
newSkeleton: userId
	self show: 'New skeleton'! !

!KinectServerMorph methodsFor: 'kinect events' stamp: 'Richo 1/17/2011 14:31'!
newUser: userId
	self show: 'New user'.! !

!KinectServerMorph methodsFor: 'kinect events' stamp: 'Richo 1/17/2011 14:31'!
poseDetected: userId
	self show: 'Pose detected'! !

!KinectServerMorph methodsFor: 'updating ui' stamp: 'Richo 1/17/2011 16:45'!
position: aMorph accordingTo: aPoint3D
	"Moves the morph to the relative position given as a parameter. It also changes its size and color depending on the point's z"
	WorldState addDeferredUIMessage: [
	aMorph extent: (self height min: self width) / 5 * (1 - aPoint3D z) * (1.1@1.1);
		 color: (Color blue alpha: (1 - aPoint3D z));
		 center: self topLeft + (aPoint3D x * self width @ (aPoint3D y * self height))].
! !

!KinectServerMorph methodsFor: 'initialize' stamp: 'Richo 1/17/2011 16:48'!
positionJoints
"Moves all the joints to their initial position"
self initialJointPositions keysAndValuesDo: [:jointName :jointPosition |
	self position: (skeleton at: jointName) accordingTo: jointPosition]! !

!KinectServerMorph methodsFor: 'updating ui' stamp: 'Richo 1/17/2011 16:46'!
show: aString
	"Shows a msg in the top of the morph"
	aString isEmpty ifTrue: [informationMorph ifNotNilDo: [:m | m delete]. informationMorph := nil. ^self].
	informationMorph isNil ifTrue: [self addMorph: (informationMorph := TextMorph new)].
	informationMorph contents: aString.
	informationMorph center: self center x @ (self top + informationMorph height).
! !

!KinectServerMorph methodsFor: 'updating ui' stamp: 'Richo 1/17/2011 16:46'!
start
""
kinectServer connect.
self borderColor: Color green; startStepping! !

!KinectServerMorph methodsFor: 'updating ui' stamp: 'Richo 1/17/2011 16:46'!
stop
""
kinectServer disconnect.
self positionJoints.
self borderColor: Color red; startStepping.
! !

!KinectServerMorph methodsFor: 'kinect events' stamp: 'Richo 1/17/2011 16:48'!
updateJoint: jointName position: jointPosition user: userId
	self show: '';
		 position: (skeleton at: jointName ifAbsent: [^self]) accordingTo: jointPosition! !
